@@ -1,12 +1,20 @@
+#define NPY_NO_DEPRECATED_API NPY_API_VERSION
 #include "Python.h"
 #include "structmember.h"
-#include "numpy/noprefix.h"
+#include "numpy/arrayobject.h"
+#include "numpy/npy_3kcompat.h"
 #include "npy_config.h"
+#include "numpy/ufuncobject.h"
+#include "string.h"
 
-static intp
-incr_slot_(double x, double *bins, intp lbins)
+#if (PY_VERSION_HEX < 0x02060000)
+#define Py_TYPE(o)    (((PyObject*)(o))->ob_type)
+#endif
+
+static npy_intp
+incr_slot_(double x, double *bins, npy_intp lbins)
 {
-    intp i;
+    npy_intp i;
 
     for ( i = 0; i < lbins; i ++ ) {
         if ( x < bins [i] ) {
@@ -16,10 +24,10 @@ incr_slot_(double x, double *bins, intp lbins)
     return lbins;
 }
 
-static intp
-decr_slot_(double x, double * bins, intp lbins)
+static npy_intp
+decr_slot_(double x, double * bins, npy_intp lbins)
 {
-    intp i;
+    npy_intp i;
 
     for ( i = lbins - 1; i >= 0; i -- ) {
         if (x < bins [i]) {
@@ -29,8 +37,39 @@ decr_slot_(double x, double * bins, intp lbins)
     return 0;
 }
 
+static npy_intp
+incr_slot_right_(double x, double *bins, npy_intp lbins)
+{
+    npy_intp i;
+
+    for ( i = 0; i < lbins; i ++ ) {
+        if ( x <= bins [i] ) {
+            return i;
+        }
+    }
+    return lbins;
+}
+
+static npy_intp
+decr_slot_right_(double x, double * bins, npy_intp lbins)
+{
+    npy_intp i;
+
+    for ( i = lbins - 1; i >= 0; i -- ) {
+        if (x <= bins [i]) {
+            return i + 1;
+        }
+    }
+    return 0;
+}
+
+/**
+ * Returns -1 if the array is monotonic decreasing,
+ * +1 if the array is monotonic increasing,
+ * and 0 if the array is not monotonic.
+ */
 static int
-monotonic_(double * a, int lena)
+check_array_monotonic(double * a, int lena)
 {
     int i;
 
@@ -57,11 +96,11 @@ monotonic_(double * a, int lena)
 
 
 /* find the index of the maximum element of an integer array */
-static intp
-mxx (intp *i , intp len)
+static npy_intp
+mxx (npy_intp *i , npy_intp len)
 {
-    intp mx = 0, max = i[0];
-    intp j;
+    npy_intp mx = 0, max = i[0];
+    npy_intp j;
 
     for ( j = 1; j < len; j ++ ) {
         if ( i [j] > max ) {
@@ -73,11 +112,11 @@ mxx (intp *i , intp len)
 }
 
 /* find the index of the minimum element of an integer array */
-static intp
-mnx (intp *i , intp len)
+static npy_intp
+mnx (npy_intp *i , npy_intp len)
 {
-    intp mn = 0, min = i [0];
-    intp j;
+    npy_intp mn = 0, min = i [0];
+    npy_intp j;
 
     for ( j = 1; j < len; j ++ )
         if ( i [j] < min )
@@ -105,8 +144,8 @@ arr_bincount(PyObject *NPY_UNUSED(self), PyObject *args, PyObject *kwds)
 {
     PyArray_Descr *type;
     PyObject *list = NULL, *weight=Py_None, *mlength=Py_None;
-    PyObject *lst=NULL, *ans=NULL, *wts=NULL;
-    intp *numbers, *ians, len , mxi, mni, ans_size, minlength;
+    PyArrayObject *lst=NULL, *ans=NULL, *wts=NULL;
+    npy_intp *numbers, *ians, len , mxi, mni, ans_size, minlength;
     int i;
     double *weights , *dans;
     static char *kwlist[] = {"list", "weights", "minlength", NULL};
@@ -115,16 +154,30 @@ arr_bincount(PyObject *NPY_UNUSED(self), PyObject *args, PyObject *kwds)
                 kwlist, &list, &weight, &mlength)) {
             goto fail;
     }
-    if (!(lst = PyArray_ContiguousFromAny(list, PyArray_INTP, 1, 1))) {
-            goto fail;
-    }
-    len = PyArray_SIZE(lst);
-    if (len < 1) {
-        PyErr_SetString(PyExc_ValueError,
-                "The first argument cannot be empty.");
+
+    lst = (PyArrayObject *)PyArray_ContiguousFromAny(list, NPY_INTP, 1, 1);
+    if (lst == NULL) {
         goto fail;
     }
-    numbers = (intp *) PyArray_DATA(lst);
+    len = PyArray_SIZE(lst);
+    type = PyArray_DescrFromType(NPY_INTP);
+
+    /* handle empty list */
+    if (len < 1) {
+        if (mlength == Py_None) {
+            minlength = 0;
+        }
+        else if (!(minlength = PyArray_PyIntAsIntp(mlength))) {
+            goto fail;
+        }
+        if (!(ans = (PyArrayObject *)PyArray_Zeros(1, &minlength, type, 0))){
+            goto fail;
+        }
+        Py_DECREF(lst);
+        return (PyObject *)ans;
+    }
+
+    numbers = (npy_intp *) PyArray_DATA(lst);
     mxi = mxx(numbers, len);
     mni = mnx(numbers, len);
     if (numbers[mni] < 0) {
@@ -147,18 +200,22 @@ arr_bincount(PyObject *NPY_UNUSED(self), PyObject *args, PyObject *kwds)
             ans_size = minlength;
         }
     }
-    type = PyArray_DescrFromType(PyArray_INTP);
     if (weight == Py_None) {
-        if (!(ans = PyArray_Zeros(1, &ans_size, type, 0))) {
+        ans = (PyArrayObject *)PyArray_Zeros(1, &ans_size, type, 0);
+        if (ans == NULL) {
             goto fail;
         }
-        ians = (intp *)(PyArray_DATA(ans));
+        ians = (npy_intp *)(PyArray_DATA(ans));
+        NPY_BEGIN_ALLOW_THREADS;
         for (i = 0; i < len; i++)
             ians [numbers [i]] += 1;
+        NPY_END_ALLOW_THREADS;
         Py_DECREF(lst);
     }
     else {
-        if (!(wts = PyArray_ContiguousFromAny(weight, PyArray_DOUBLE, 1, 1))) {
+        wts = (PyArrayObject *)PyArray_ContiguousFromAny(
+                                                weight, NPY_DOUBLE, 1, 1);
+        if (wts == NULL) {
             goto fail;
         }
         weights = (double *)PyArray_DATA (wts);
@@ -167,18 +224,21 @@ arr_bincount(PyObject *NPY_UNUSED(self), PyObject *args, PyObject *kwds)
                     "The weights and list don't have the same length.");
             goto fail;
         }
-        type = PyArray_DescrFromType(PyArray_DOUBLE);
-        if (!(ans = PyArray_Zeros(1, &ans_size, type, 0))) {
+        type = PyArray_DescrFromType(NPY_DOUBLE);
+        ans = (PyArrayObject *)PyArray_Zeros(1, &ans_size, type, 0);
+        if (ans == NULL) {
             goto fail;
         }
-        dans = (double *)PyArray_DATA (ans);
+        dans = (double *)PyArray_DATA(ans);
+        NPY_BEGIN_ALLOW_THREADS;
         for (i = 0; i < len; i++) {
             dans[numbers[i]] += weights[i];
         }
+        NPY_END_ALLOW_THREADS;
         Py_DECREF(lst);
         Py_DECREF(wts);
     }
-    return ans;
+    return (PyObject *)ans;
 
 fail:
     Py_XDECREF(lst);
@@ -189,34 +249,43 @@ fail:
 
 
 /*
- * digitize (x, bins) returns an array of python integers the same
+ * digitize (x, bins, right=False) returns an array of python integers the same
  * length of x. The values i returned are such that bins [i - 1] <= x <
  * bins [i] if bins is monotonically increasing, or bins [i - 1] > x >=
  * bins [i] if bins is monotonically decreasing.  Beyond the bounds of
  * bins, returns either i = 0 or i = len (bins) as appropriate.
+ * if right == True the comparison is bins [i - 1] < x <= bins[i]
+ * or bins [i - 1] >= x > bins[i]
  */
 static PyObject *
 arr_digitize(PyObject *NPY_UNUSED(self), PyObject *args, PyObject *kwds)
 {
     /* self is not used */
     PyObject *ox, *obins;
-    PyObject *ax = NULL, *abins = NULL, *aret = NULL;
+    PyArrayObject *ax = NULL, *abins = NULL, *aret = NULL;
     double *dx, *dbins;
-    intp lbins, lx;             /* lengths */
-    intp *iret;
+    npy_intp lbins, lx;             /* lengths */
+    npy_intp right = 0; /* whether right or left is inclusive */
+    npy_intp *iret;
     int m, i;
-    static char *kwlist[] = {"x", "bins", NULL};
+    static char *kwlist[] = {"x", "bins", "right", NULL};
     PyArray_Descr *type;
+    char bins_non_monotonic = 0;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "OO", kwlist, &ox, &obins)) {
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "OO|i", kwlist, &ox, &obins,
+                &right)) {
         goto fail;
     }
-    type = PyArray_DescrFromType(PyArray_DOUBLE);
-    if (!(ax = PyArray_FromAny(ox, type, 1, 1, CARRAY, NULL))) {
+    type = PyArray_DescrFromType(NPY_DOUBLE);
+    ax = (PyArrayObject *)PyArray_FromAny(ox, type,
+                                        1, 1, NPY_ARRAY_CARRAY, NULL);
+    if (ax == NULL) {
         goto fail;
     }
     Py_INCREF(type);
-    if (!(abins = PyArray_FromAny(obins, type, 1, 1, CARRAY, NULL))) {
+    abins = (PyArrayObject *)PyArray_FromAny(obins, type,
+                                        1, 1, NPY_ARRAY_CARRAY, NULL);
+    if (abins == NULL) {
         goto fail;
     }
 
@@ -224,49 +293,88 @@ arr_digitize(PyObject *NPY_UNUSED(self), PyObject *args, PyObject *kwds)
     dx = (double *)PyArray_DATA(ax);
     lbins = PyArray_SIZE(abins);
     dbins = (double *)PyArray_DATA(abins);
-    if (!(aret = PyArray_SimpleNew(1, &lx, PyArray_INTP))) {
+    aret = (PyArrayObject *)PyArray_SimpleNew(1, &lx, NPY_INTP);
+    if (aret == NULL) {
         goto fail;
     }
-    iret = (intp *)PyArray_DATA(aret);
+    iret = (npy_intp *)PyArray_DATA(aret);
 
     if (lx <= 0 || lbins < 0) {
         PyErr_SetString(PyExc_ValueError,
                 "Both x and bins must have non-zero length");
             goto fail;
     }
-
+    NPY_BEGIN_ALLOW_THREADS;
     if (lbins == 1)  {
-        for (i = 0; i < lx; i++) {
-            if (dx [i] >= dbins[0]) {
-                iret[i] = 1;
-            }
-            else {
-                iret[i] = 0;
-            }
-        }
-    }
-    else {
-        m = monotonic_ (dbins, lbins);
-        if ( m == -1 ) {
-            for ( i = 0; i < lx; i ++ ) {
-                iret [i] = decr_slot_ ((double)dx[i], dbins, lbins);
-            }
-        }
-        else if ( m == 1 ) {
-            for ( i = 0; i < lx; i ++ ) {
-                iret [i] = incr_slot_ ((double)dx[i], dbins, lbins);
+        if (right == 0) {
+            for (i = 0; i < lx; i++) {
+                if (dx [i] >= dbins[0]) {
+                    iret[i] = 1;
+                }
+                else {
+                    iret[i] = 0;
+                }
             }
         }
         else {
-            PyErr_SetString(PyExc_ValueError,
-                    "The bins must be montonically increasing or decreasing");
-            goto fail;
+            for (i = 0; i < lx; i++) {
+                if (dx [i] > dbins[0]) {
+                    iret[i] = 1;
+                }
+                else {
+                    iret[i] = 0;
+                }
+            }
+
         }
     }
+    else {
+        m = check_array_monotonic(dbins, lbins);
+        if (right == 0) {
+            if ( m == -1 ) {
+                for ( i = 0; i < lx; i ++ ) {
+                    iret [i] = decr_slot_ ((double)dx[i], dbins, lbins);
+                }
+            }
+            else if ( m == 1 ) {
+                for ( i = 0; i < lx; i ++ ) {
+                    iret [i] = incr_slot_ ((double)dx[i], dbins, lbins);
+                }
+            }
+            else {
+            /* defer PyErr_SetString until after NPY_END_ALLOW_THREADS */
+                bins_non_monotonic = 1;
+            }
+        }
+        else {
+            if ( m == -1 ) {
+                for ( i = 0; i < lx; i ++ ) {
+                    iret [i] = decr_slot_right_ ((double)dx[i], dbins,
+                                                                lbins);
+                }
+            }
+            else if ( m == 1 ) {
+                for ( i = 0; i < lx; i ++ ) {
+                    iret [i] = incr_slot_right_ ((double)dx[i], dbins,
+                                                               lbins);
+                }
+            }
+            else {
+            /* defer PyErr_SetString until after NPY_END_ALLOW_THREADS */
+                bins_non_monotonic = 1;
+            }
 
+        }
+    }
+    NPY_END_ALLOW_THREADS;
+    if (bins_non_monotonic) {
+        PyErr_SetString(PyExc_ValueError,
+                "The bins must be monotonically increasing or decreasing");
+        goto fail;
+    }
     Py_DECREF(ax);
     Py_DECREF(abins);
-    return aret;
+    return (PyObject *)aret;
 
 fail:
     Py_XDECREF(ax);
@@ -279,6 +387,55 @@ fail:
 
 static char arr_insert__doc__[] = "Insert vals sequentially into equivalent 1-d positions indicated by mask.";
 
+/*
+ * Insert values from an input array into an output array, at positions
+ * indicated by a mask. If the arrays are of dtype object (indicated by
+ * the objarray flag), take care of reference counting.
+ *
+ * This function implements the copying logic of arr_insert() defined
+ * below.
+ */
+static void
+arr_insert_loop(char *mptr, char *vptr, char *input_data, char *zero,
+                char *avals_data, int melsize, int delsize, int objarray,
+                int totmask, int numvals, int nd, npy_intp *instrides,
+                npy_intp *inshape)
+{
+    int mindx, rem_indx, indx, i, copied;
+
+    /*
+     * Walk through mask array, when non-zero is encountered
+     * copy next value in the vals array to the input array.
+     * If we get through the value array, repeat it as necessary.
+     */
+    copied = 0;
+    for (mindx = 0; mindx < totmask; mindx++) {
+        if (memcmp(mptr,zero,melsize) != 0) {
+            /* compute indx into input array */
+            rem_indx = mindx;
+            indx = 0;
+            for (i = nd - 1; i > 0; --i) {
+                indx += (rem_indx % inshape[i]) * instrides[i];
+                rem_indx /= inshape[i];
+            }
+            indx += rem_indx * instrides[0];
+            /* fprintf(stderr, "mindx = %d, indx=%d\n", mindx, indx); */
+            /* Copy value element over to input array */
+            memcpy(input_data+indx,vptr,delsize);
+            if (objarray) {
+                Py_INCREF(*((PyObject **)vptr));
+            }
+            vptr += delsize;
+            copied += 1;
+            /* If we move past value data.  Reset */
+            if (copied >= numvals) {
+                vptr = avals_data;
+            }
+        }
+        mptr += melsize;
+    }
+}
+
 /*
  * Returns input array with values inserted sequentially into places
  * indicated by the mask
@@ -290,9 +447,8 @@ arr_insert(PyObject *NPY_UNUSED(self), PyObject *args, PyObject *kwdict)
     PyArrayObject *ainput = NULL, *amask = NULL, *avals = NULL, *tmp = NULL;
     int numvals, totmask, sameshape;
     char *input_data, *mptr, *vptr, *zero = NULL;
-    int melsize, delsize, copied, nd;
-    intp *instrides, *inshape;
-    int mindx, rem_indx, indx, i, k, objarray;
+    int melsize, delsize, nd, objarray, k;
+    npy_intp *instrides, *inshape;
 
     static char *kwlist[] = {"input", "mask", "vals", NULL};
 
@@ -302,13 +458,13 @@ arr_insert(PyObject *NPY_UNUSED(self), PyObject *args, PyObject *kwdict)
         goto fail;
     }
 
-    amask = (PyArrayObject *) PyArray_FROM_OF(mask, CARRAY);
+    amask = (PyArrayObject *)PyArray_FROM_OF(mask, NPY_ARRAY_CARRAY);
     if (amask == NULL) {
         goto fail;
     }
     /* Cast an object array */
-    if (amask->descr->type_num == PyArray_OBJECT) {
-        tmp = (PyArrayObject *)PyArray_Cast(amask, PyArray_INTP);
+    if (PyArray_DESCR(amask)->type_num == NPY_OBJECT) {
+        tmp = (PyArrayObject *)PyArray_Cast(amask, NPY_INTP);
         if (tmp == NULL) {
             goto fail;
         }
@@ -317,16 +473,16 @@ arr_insert(PyObject *NPY_UNUSED(self), PyObject *args, PyObject *kwdict)
     }
 
     sameshape = 1;
-    if (amask->nd == ainput->nd) {
-        for (k = 0; k < amask->nd; k++) {
-            if (amask->dimensions[k] != ainput->dimensions[k]) {
+    if (PyArray_NDIM(amask) == PyArray_NDIM(ainput)) {
+        for (k = 0; k < PyArray_NDIM(amask); k++) {
+            if (PyArray_DIMS(amask)[k] != PyArray_DIMS(ainput)[k]) {
                 sameshape = 0;
             }
         }
     }
     else {
         /* Test to see if amask is 1d */
-        if (amask->nd != 1) {
+        if (PyArray_NDIM(amask) != 1) {
             sameshape = 0;
         }
         else if ((PyArray_SIZE(ainput)) != PyArray_SIZE(amask)) {
@@ -339,22 +495,23 @@ arr_insert(PyObject *NPY_UNUSED(self), PyObject *args, PyObject *kwdict)
         goto fail;
     }
 
-    avals = (PyArrayObject *)PyArray_FromObject(vals, ainput->descr->type_num, 0, 1);
+    avals = (PyArrayObject *)PyArray_FromObject(vals,
+                                        PyArray_DESCR(ainput)->type_num, 0, 1);
     if (avals == NULL) {
         goto fail;
     }
     numvals = PyArray_SIZE(avals);
-    nd = ainput->nd;
-    input_data = ainput->data;
-    mptr = amask->data;
-    melsize = amask->descr->elsize;
-    vptr = avals->data;
-    delsize = avals->descr->elsize;
+    nd = PyArray_NDIM(ainput);
+    input_data = PyArray_DATA(ainput);
+    mptr = PyArray_DATA(amask);
+    melsize = PyArray_DESCR(amask)->elsize;
+    vptr = PyArray_DATA(avals);
+    delsize = PyArray_DESCR(avals)->elsize;
     zero = PyArray_Zero(amask);
     if (zero == NULL) {
         goto fail;
     }
-    objarray = (ainput->descr->type_num == PyArray_OBJECT);
+    objarray = (PyArray_DESCR(ainput)->type_num == NPY_OBJECT);
 
     /* Handle zero-dimensional case separately */
     if (nd == 0) {
@@ -373,39 +530,22 @@ arr_insert(PyObject *NPY_UNUSED(self), PyObject *args, PyObject *kwdict)
         return Py_None;
     }
 
-    /*
-     * Walk through mask array, when non-zero is encountered
-     * copy next value in the vals array to the input array.
-     * If we get through the value array, repeat it as necessary.
-     */
     totmask = (int) PyArray_SIZE(amask);
-    copied = 0;
-    instrides = ainput->strides;
-    inshape = ainput->dimensions;
-    for (mindx = 0; mindx < totmask; mindx++) {
-        if (memcmp(mptr,zero,melsize) != 0) {
-            /* compute indx into input array */
-            rem_indx = mindx;
-            indx = 0;
-            for(i = nd - 1; i > 0; --i) {
-                indx += (rem_indx % inshape[i]) * instrides[i];
-                rem_indx /= inshape[i];
-            }
-            indx += rem_indx * instrides[0];
-            /* fprintf(stderr, "mindx = %d, indx=%d\n", mindx, indx); */
-            /* Copy value element over to input array */
-            memcpy(input_data+indx,vptr,delsize);
-            if (objarray) {
-                Py_INCREF(*((PyObject **)vptr));
-            }
-            vptr += delsize;
-            copied += 1;
-            /* If we move past value data.  Reset */
-            if (copied >= numvals) {
-                vptr = avals->data;
-            }
-        }
-        mptr += melsize;
+    instrides = PyArray_STRIDES(ainput);
+    inshape = PyArray_DIMS(ainput);
+    if (objarray) {
+        /* object array, need to refcount, can't release the GIL */
+        arr_insert_loop(mptr, vptr, input_data, zero, PyArray_DATA(avals),
+                        melsize, delsize, objarray, totmask, numvals, nd,
+                        instrides, inshape);
+    }
+    else {
+        /* No increfs take place in arr_insert_loop, so release the GIL */
+        NPY_BEGIN_ALLOW_THREADS;
+        arr_insert_loop(mptr, vptr, input_data, zero, PyArray_DATA(avals),
+                        melsize, delsize, objarray, totmask, numvals, nd,
+                        instrides, inshape);
+        NPY_END_ALLOW_THREADS;
     }
 
     Py_DECREF(amask);
@@ -465,7 +605,7 @@ arr_interp(PyObject *NPY_UNUSED(self), PyObject *args, PyObject *kwdict)
     PyObject *fp, *xp, *x;
     PyObject *left = NULL, *right = NULL;
     PyArrayObject *afp = NULL, *axp = NULL, *ax = NULL, *af = NULL;
-    npy_intp i, lenx, lenxp, indx;
+    npy_intp i, lenx, lenxp;
     double lval, rval;
     double *dy, *dx, *dz, *dres, *slopes;
 
@@ -476,31 +616,32 @@ arr_interp(PyObject *NPY_UNUSED(self), PyObject *args, PyObject *kwdict)
         return NULL;
     }
 
-    afp = (NPY_AO*)PyArray_ContiguousFromAny(fp, NPY_DOUBLE, 1, 1);
+    afp = (PyArrayObject *)PyArray_ContiguousFromAny(fp, NPY_DOUBLE, 1, 1);
     if (afp == NULL) {
         return NULL;
     }
-    axp = (NPY_AO*)PyArray_ContiguousFromAny(xp, NPY_DOUBLE, 1, 1);
+    axp = (PyArrayObject *)PyArray_ContiguousFromAny(xp, NPY_DOUBLE, 1, 1);
     if (axp == NULL) {
         goto fail;
     }
-    ax = (NPY_AO*)PyArray_ContiguousFromAny(x, NPY_DOUBLE, 1, 0);
+    ax = (PyArrayObject *)PyArray_ContiguousFromAny(x, NPY_DOUBLE, 1, 0);
     if (ax == NULL) {
         goto fail;
     }
-    lenxp = axp->dimensions[0];
+    lenxp = PyArray_DIMS(axp)[0];
     if (lenxp == 0) {
         PyErr_SetString(PyExc_ValueError,
                 "array of sample points is empty");
         goto fail;
     }
-    if (afp->dimensions[0] != lenxp) {
+    if (PyArray_DIMS(afp)[0] != lenxp) {
         PyErr_SetString(PyExc_ValueError,
                 "fp and xp are not of the same length.");
         goto fail;
     }
 
-    af = (NPY_AO*)PyArray_SimpleNew(ax->nd, ax->dimensions, NPY_DOUBLE);
+    af = (PyArrayObject *)PyArray_SimpleNew(PyArray_NDIM(ax),
+                                        PyArray_DIMS(ax), NPY_DOUBLE);
     if (af == NULL) {
         goto fail;
     }
@@ -531,27 +672,57 @@ arr_interp(PyObject *NPY_UNUSED(self), PyObject *args, PyObject *kwdict)
         }
     }
 
-    slopes = (double *) PyDataMem_NEW((lenxp - 1)*sizeof(double));
-    for (i = 0; i < lenxp - 1; i++) {
-        slopes[i] = (dy[i + 1] - dy[i])/(dx[i + 1] - dx[i]);
+    /* only pre-calculate slopes if there are relatively few of them. */
+    if (lenxp <= lenx) {
+        slopes = (double *) PyArray_malloc((lenxp - 1)*sizeof(double));
+        if (! slopes) {
+            goto fail;
+        }
+        NPY_BEGIN_ALLOW_THREADS;
+        for (i = 0; i < lenxp - 1; i++) {
+            slopes[i] = (dy[i + 1] - dy[i])/(dx[i + 1] - dx[i]);
+        }
+        for (i = 0; i < lenx; i++) {
+            npy_intp j = binary_search(dz[i], dx, lenxp);
+
+            if (j == -1) {
+                dres[i] = lval;
+            }
+            else if (j == lenxp - 1) {
+                dres[i] = dy[j];
+            }
+            else if (j == lenxp) {
+                dres[i] = rval;
+            }
+            else {
+                dres[i] = slopes[j]*(dz[i] - dx[j]) + dy[j];
+            }
+        }
+        NPY_END_ALLOW_THREADS;
+        PyArray_free(slopes);
     }
-    for (i = 0; i < lenx; i++) {
-        indx = binary_search(dz[i], dx, lenxp);
-        if (indx == -1) {
-            dres[i] = lval;
-        }
-        else if (indx == lenxp - 1) {
-            dres[i] = dy[indx];
-        }
-        else if (indx == lenxp) {
-            dres[i] = rval;
-        }
-        else {
-            dres[i] = slopes[indx]*(dz[i] - dx[indx]) + dy[indx];
+    else {
+        NPY_BEGIN_ALLOW_THREADS;
+        for (i = 0; i < lenx; i++) {
+            npy_intp j = binary_search(dz[i], dx, lenxp);
+
+            if (j == -1) {
+                dres[i] = lval;
+            }
+            else if (j == lenxp - 1) {
+                dres[i] = dy[j];
+            }
+            else if (j == lenxp) {
+                dres[i] = rval;
+            }
+            else {
+                double slope = (dy[j + 1] - dy[j])/(dx[j + 1] - dx[j]);
+                dres[i] = slope*(dz[i] - dx[j]) + dy[j];
+            }
         }
+        NPY_END_ALLOW_THREADS;
     }
 
-    PyDataMem_FREE(slopes);
     Py_DECREF(afp);
     Py_DECREF(axp);
     Py_DECREF(ax);
@@ -621,8 +792,11 @@ ravel_multi_index_loop(int ravel_ndim, npy_intp *ravel_dims,
                         char **coords, npy_intp *coords_strides)
 {
     int i;
+    char invalid;
     npy_intp j, m;
 
+    NPY_BEGIN_ALLOW_THREADS;
+    invalid = 0;
     while (count--) {
         npy_intp raveled = 0;
         for (i = 0; i < ravel_ndim; ++i) {
@@ -630,35 +804,34 @@ ravel_multi_index_loop(int ravel_ndim, npy_intp *ravel_dims,
             j = *(npy_intp *)coords[i];
             switch (modes[i]) {
                 case NPY_RAISE:
-                    if(j < 0 || j>=m) {
-                        PyErr_SetString(PyExc_ValueError,
-                              "invalid entry in coordinates array");
-                        return NPY_FAIL;
+                    if (j < 0 || j >= m) {
+                        invalid = 1;
+                        goto end_while;
                     }
                     break;
                 case NPY_WRAP:
-                    if(j < 0) {
+                    if (j < 0) {
                         j += m;
-                        if(j < 0) {
-                            j = j%m;
-                            if(j != 0) {
+                        if (j < 0) {
+                            j = j % m;
+                            if (j != 0) {
                                 j += m;
                             }
                         }
                     }
-                    else if(j >= m) {
+                    else if (j >= m) {
                         j -= m;
-                        if(j >= m) {
-                            j = j%m;
+                        if (j >= m) {
+                            j = j % m;
                         }
                     }
                     break;
                 case NPY_CLIP:
-                    if(j < 0) {
+                    if (j < 0) {
                         j = 0;
                     }
-                    else if(j >= m) {
-                        j = m-1;
+                    else if (j >= m) {
+                        j = m - 1;
                     }
                     break;
 
@@ -670,7 +843,13 @@ ravel_multi_index_loop(int ravel_ndim, npy_intp *ravel_dims,
         *(npy_intp *)coords[ravel_ndim] = raveled;
         coords[ravel_ndim] += coords_strides[ravel_ndim];
     }
-
+end_while:
+    NPY_END_ALLOW_THREADS;
+    if (invalid) {
+        PyErr_SetString(PyExc_ValueError,
+              "invalid entry in coordinates array");
+        return NPY_FAIL;
+    }
     return NPY_SUCCEED;
 }
 
@@ -683,7 +862,7 @@ arr_ravel_multi_index(PyObject *self, PyObject *args, PyObject *kwds)
     PyArrayObject *ret = NULL;
     PyArray_Dims dimensions={0,0};
     npy_intp ravel_strides[NPY_MAXDIMS];
-    PyArray_ORDER order = NPY_CORDER;
+    NPY_ORDER order = NPY_CORDER;
     NPY_CLIPMODE modes[NPY_MAXDIMS];
 
     PyArrayObject *op[NPY_MAXARGS];
@@ -697,7 +876,7 @@ arr_ravel_multi_index(PyObject *self, PyObject *args, PyObject *kwds)
     memset(op, 0, sizeof(op));
     dtype[0] = NULL;
 
-    if(!PyArg_ParseTupleAndKeywords(args, kwds,
+    if (!PyArg_ParseTupleAndKeywords(args, kwds,
                         "OO&|OO&:ravel_multi_index", kwlist,
                      &coords0,
                      PyArray_IntpConverter, &dimensions,
@@ -712,7 +891,7 @@ arr_ravel_multi_index(PyObject *self, PyObject *args, PyObject *kwds)
         goto fail;
     }
 
-    if(!PyArray_ConvertClipmodeSequence(mode0, modes, dimensions.len)) {
+    if (!PyArray_ConvertClipmodeSequence(mode0, modes, dimensions.len)) {
        goto fail;
     }
 
@@ -821,14 +1000,16 @@ unravel_index_loop_corder(int unravel_ndim, npy_intp *unravel_dims,
                         npy_intp *coords)
 {
     int i;
+    char invalid;
     npy_intp val;
 
+    NPY_BEGIN_ALLOW_THREADS;
+    invalid = 0;
     while (count--) {
         val = *(npy_intp *)indices;
         if (val < 0 || val >= unravel_size) {
-            PyErr_SetString(PyExc_ValueError,
-                  "invalid entry in index array");
-            return NPY_FAIL;
+            invalid = 1;
+            break;
         }
         for (i = unravel_ndim-1; i >= 0; --i) {
             coords[i] = val % unravel_dims[i];
@@ -837,7 +1018,12 @@ unravel_index_loop_corder(int unravel_ndim, npy_intp *unravel_dims,
         coords += unravel_ndim;
         indices += indices_stride;
     }
-
+    NPY_END_ALLOW_THREADS;
+    if (invalid) {
+        PyErr_SetString(PyExc_ValueError,
+              "invalid entry in index array");
+        return NPY_FAIL;
+    }
     return NPY_SUCCEED;
 }
 
@@ -849,14 +1035,16 @@ unravel_index_loop_forder(int unravel_ndim, npy_intp *unravel_dims,
                         npy_intp *coords)
 {
     int i;
+    char invalid;
     npy_intp val;
 
+    NPY_BEGIN_ALLOW_THREADS;
+    invalid = 0;
     while (count--) {
         val = *(npy_intp *)indices;
         if (val < 0 || val >= unravel_size) {
-            PyErr_SetString(PyExc_ValueError,
-                  "invalid entry in index array");
-            return NPY_FAIL;
+            invalid = 1;
+            break;
         }
         for (i = 0; i < unravel_ndim; ++i) {
             *coords++ = val % unravel_dims[i];
@@ -864,7 +1052,12 @@ unravel_index_loop_forder(int unravel_ndim, npy_intp *unravel_dims,
         }
         indices += indices_stride;
     }
-
+    NPY_END_ALLOW_THREADS;
+    if (invalid) {
+        PyErr_SetString(PyExc_ValueError,
+              "invalid entry in index array");
+        return NPY_FAIL;
+    }
     return NPY_SUCCEED;
 }
 
@@ -877,7 +1070,7 @@ arr_unravel_index(PyObject *self, PyObject *args, PyObject *kwds)
     PyArrayObject *indices = NULL;
     PyArray_Descr *dtype = NULL;
     PyArray_Dims dimensions={0,0};
-    PyArray_ORDER order = PyArray_CORDER;
+    NPY_ORDER order = NPY_CORDER;
     npy_intp unravel_size;
 
     NpyIter *iter = NULL;
@@ -886,7 +1079,7 @@ arr_unravel_index(PyObject *self, PyObject *args, PyObject *kwds)
 
     char *kwlist[] = {"indices", "dims", "order", NULL};
 
-    if(!PyArg_ParseTupleAndKeywords(args, kwds, "OO&|O&:unravel_index",
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "OO&|O&:unravel_index",
                     kwlist,
                     &indices0,
                     PyArray_IntpConverter, &dimensions,
@@ -902,10 +1095,10 @@ arr_unravel_index(PyObject *self, PyObject *args, PyObject *kwds)
 
     unravel_size = PyArray_MultiplyList(dimensions.ptr, dimensions.len);
 
-    if(!PyArray_Check(indices0)) {
+    if (!PyArray_Check(indices0)) {
         indices = (PyArrayObject*)PyArray_FromAny(indices0,
                                                     NULL, 0, 0, 0, NULL);
-        if(indices == NULL) {
+        if (indices == NULL) {
             goto fail;
         }
     }
@@ -1038,7 +1231,10 @@ arr_unravel_index(PyObject *self, PyObject *args, PyObject *kwds)
             goto fail;
         }
         Py_INCREF(ret_arr);
-        view->base = (PyObject *)ret_arr;
+        if (PyArray_SetBaseObject(view, (PyObject *)ret_arr) < 0) {
+            Py_DECREF(view);
+            goto fail;
+        }
         PyTuple_SET_ITEM(ret_tuple, i, PyArray_Return(view));
     }
 
@@ -1096,8 +1292,8 @@ arr_add_docstring(PyObject *NPY_UNUSED(dummy), PyObject *args)
     docstr = PyString_AS_STRING(str);
 #endif
 
-#define _TESTDOC1(typebase) (obj->ob_type == &Py##typebase##_Type)
-#define _TESTDOC2(typebase) (obj->ob_type == Py##typebase##_TypePtr)
+#define _TESTDOC1(typebase) (Py_TYPE(obj) == &Py##typebase##_Type)
+#define _TESTDOC2(typebase) (Py_TYPE(obj) == Py##typebase##_TypePtr)
 #define _ADDDOC(typebase, doc, name) do {                               \
         Py##typebase##Object *new = (Py##typebase##Object *)obj;        \
         if (!(doc)) {                                                   \
@@ -1153,6 +1349,48 @@ arr_add_docstring(PyObject *NPY_UNUSED(dummy), PyObject *args)
 }
 
 
+/* docstring in numpy.add_newdocs.py */
+static PyObject *
+add_newdoc_ufunc(PyObject *NPY_UNUSED(dummy), PyObject *args)
+{
+    PyUFuncObject *ufunc;
+    PyObject *str;
+    char *docstr, *newdocstr;
+
+#if defined(NPY_PY3K)
+    if (!PyArg_ParseTuple(args, "O!O!", &PyUFunc_Type, &ufunc,
+                                        &PyUnicode_Type, &str)) {
+        return NULL;
+    }
+    docstr = PyBytes_AS_STRING(PyUnicode_AsUTF8String(str));
+#else
+    if (!PyArg_ParseTuple(args, "O!O!", &PyUFunc_Type, &ufunc,
+                                         &PyString_Type, &str)) {
+         return NULL;
+    }
+    docstr = PyString_AS_STRING(str);
+#endif
+
+    if (NULL != ufunc->doc) {
+        PyErr_SetString(PyExc_ValueError,
+                "Cannot change docstring of ufunc with non-NULL docstring");
+        return NULL;
+    }
+
+    /*
+     * This introduces a memory leak, as the memory allocated for the doc
+     * will not be freed even if the ufunc itself is deleted. In practice
+     * this should not be a problem since the user would have to
+     * repeatedly create, document, and throw away ufuncs.
+     */
+    newdocstr = malloc(strlen(docstr) + 1);
+    strcpy(newdocstr, docstr);
+    ufunc->doc = newdocstr;
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 /*  PACKBITS
  *
  *  This function packs binary (0 or 1) 1-bit per pixel arrays
@@ -1246,9 +1484,9 @@ static PyObject *
 pack_or_unpack_bits(PyObject *input, int axis, int unpack)
 {
     PyArrayObject *inp;
-    PyObject *new = NULL;
-    PyObject *out = NULL;
-    npy_intp outdims[MAX_DIMS];
+    PyArrayObject *new = NULL;
+    PyArrayObject *out = NULL;
+    npy_intp outdims[NPY_MAXDIMS];
     int i;
     void (*thefunc)(void *, int, npy_intp, npy_intp, void *, npy_intp, npy_intp);
     PyArrayIterObject *it, *ot;
@@ -1271,25 +1509,25 @@ pack_or_unpack_bits(PyObject *input, int axis, int unpack)
         goto fail;
     }
 
-    new = PyArray_CheckAxis(inp, &axis, 0);
+    new = (PyArrayObject *)PyArray_CheckAxis(inp, &axis, 0);
     Py_DECREF(inp);
     if (new == NULL) {
         return NULL;
     }
     /* Handle zero-dim array separately */
     if (PyArray_SIZE(new) == 0) {
-        return PyArray_Copy((PyArrayObject *)new);
+        return PyArray_Copy(new);
     }
 
     if (PyArray_NDIM(new) == 0) {
         if (unpack) {
             /* Handle 0-d array by converting it to a 1-d array */
-            PyObject *temp;
+            PyArrayObject *temp;
             PyArray_Dims newdim = {NULL, 1};
             npy_intp shape = 1;
 
             newdim.ptr = &shape;
-            temp = PyArray_Newshape((PyArrayObject *)new, &newdim, NPY_CORDER);
+            temp = (PyArrayObject *)PyArray_Newshape(new, &newdim, NPY_CORDER);
             if (temp == NULL) {
                 goto fail;
             }
@@ -1297,8 +1535,8 @@ pack_or_unpack_bits(PyObject *input, int axis, int unpack)
             new = temp;
         }
         else {
-            ubyte *optr, *iptr;
-            out = PyArray_New(new->ob_type, 0, NULL, NPY_UBYTE,
+            char *optr, *iptr;
+            out = (PyArrayObject *)PyArray_New(Py_TYPE(new), 0, NULL, NPY_UBYTE,
                     NULL, NULL, 0, 0, NULL);
             if (out == NULL) {
                 goto fail;
@@ -1338,8 +1576,9 @@ pack_or_unpack_bits(PyObject *input, int axis, int unpack)
     }
 
     /* Create output array */
-    out = PyArray_New(new->ob_type, PyArray_NDIM(new), outdims, PyArray_UBYTE,
-            NULL, NULL, 0, PyArray_ISFORTRAN(new), NULL);
+    out = (PyArrayObject *)PyArray_New(Py_TYPE(new),
+                        PyArray_NDIM(new), outdims, NPY_UBYTE,
+                        NULL, NULL, 0, PyArray_ISFORTRAN(new), NULL);
     if (out == NULL) {
         goto fail;
     }
@@ -1365,7 +1604,7 @@ pack_or_unpack_bits(PyObject *input, int axis, int unpack)
 
 finish:
     Py_DECREF(new);
-    return out;
+    return (PyObject *)out;
 
 fail:
     Py_XDECREF(new);
@@ -1418,6 +1657,8 @@ static struct PyMethodDef methods[] = {
         METH_VARARGS | METH_KEYWORDS, NULL},
     {"add_docstring", (PyCFunction)arr_add_docstring,
         METH_VARARGS, NULL},
+    {"add_newdoc_ufunc", (PyCFunction)add_newdoc_ufunc,
+        METH_VARARGS, NULL},
     {"packbits", (PyCFunction)io_pack,
         METH_VARARGS | METH_KEYWORDS, NULL},
     {"unpackbits", (PyCFunction)io_unpack,
@@ -1437,17 +1678,17 @@ define_types(void)
     if (myobj == NULL) {
         return;
     }
-    PyGetSetDescr_TypePtr = myobj->ob_type;
+    PyGetSetDescr_TypePtr = Py_TYPE(myobj);
     myobj = PyDict_GetItemString(tp_dict, "alignment");
     if (myobj == NULL) {
         return;
     }
-    PyMemberDescr_TypePtr = myobj->ob_type;
+    PyMemberDescr_TypePtr = Py_TYPE(myobj);
     myobj = PyDict_GetItemString(tp_dict, "newbyteorder");
     if (myobj == NULL) {
         return;
     }
-    PyMethodDescr_TypePtr = myobj->ob_type;
+    PyMethodDescr_TypePtr = Py_TYPE(myobj);
     return;
 }
 
@@ -1487,6 +1728,7 @@ init_compiled_base(void)
 
     /* Import the array objects */
     import_array();
+    import_umath();
 
     /* Add some symbolic constants to the module */
     d = PyModule_GetDict(m);
